@@ -27,11 +27,11 @@ def main(map_function):
     gradFD.setupTemplates(cache)
     hof = evo.run(cache)
 
-    multiprocessing.get_logger().info("altScoress %s", cache.altScores)
+    multiprocessing.get_logger().info("altScores %s", cache.altScores)
     multiprocessing.get_logger().info("altScoreNames %s", cache.altScoreNames)
 
     if cache.altScores:
-        for name in cache.altFeatureNames:
+        for name in cache.altScoreNames:
             json_path = util.setupAltFeature(cache, name)
             setup(cache, json_path, map_function)
             hof = evo.run(cache)
@@ -143,7 +143,7 @@ def print_version():
         ("scikit-learn", "0.24.2"),
         ("jstyleson", "0.2.0"),
         ('filelock', "3.0.12"),
-        ('pymoo', '0.4.2.2')
+        ('pymoo', '0.6.0.1')
     ]
 
     for module, version_tested in modules:
