@@ -11,7 +11,7 @@ name = "SSE"
 def get_settings(feature):
     settings = Dict()
     settings.adaptive = False
-    settings.badScore = -sys.float_info.max
+    settings.badScore = sys.float_info.max
     settings.meta_mask = True
     settings.count = 1
     settings.graph_der = 0
@@ -34,16 +34,13 @@ def run(sim_data, feature):
 
     return (
         [
-            -sse,
+            sse,
         ],
         sse,
         len(sim_data_values),
         sim_time_values,
         sim_data_values,
-        exp_data_values,
-        [
-            sse,
-        ],
+        exp_data_values
     )
 
 
